/*
 * Lars Wirzenius <liw@iki.fi>
 */

#define MAX_HASH 10007


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>


struct word {
    char *word;
    unsigned long hash;
    long count;
    struct word *next;
};


static unsigned long hash(char *s)
{
    unsigned long h;

    for (h = 0; *s != '\0'; ++s)
	h = (h * 33) | *s;
    return h;
}


static struct word *allocate(char *word)
{
    struct word *w;
    
    w = malloc(sizeof(*w));
    if (w != NULL)
    	w->word = malloc(strlen(word) + 1);
    if (w == NULL || w->word == NULL) {
    	perror("malloc");
	exit(EXIT_FAILURE);
    }
    strcpy(w->word, word);
    w->count = 1;
    w->hash = hash(word);
    w->next = NULL;
    return w;
}


static void add_count(struct word **hashtab, char *word)
{
    struct word *w;
    unsigned long h;
    int i;
    
    h = hash(word);
    i = h % MAX_HASH;

    for (w = hashtab[i]; w != NULL; w = w->next)
    	if (h == w->hash && strcmp(word, w->word) == 0)
	    break;

    if (w == NULL) {
    	w = allocate(word);
	w->next = hashtab[i];
	hashtab[i] = w;
    } else {
	++w->count;
    }
}


static int comes_before(struct word *a, struct word *b)
{
    return a->count > b->count || 
    	   (a->count == b->count && strcmp(a->word, b->word) < 0);
}

static int insert(struct word **tab, int len, int max, struct word *w)
{
    int i;
    
    if (len == 0) {
	tab[0] = w;
	return 1;
    } else if (len < max) {
	for (i = len; i > 0 && comes_before(w, tab[i-1]); --i)
	    tab[i] = tab[i-1];
	tab[i] = w;
    	return len + 1;
    } else if (comes_before(w, tab[len-1])) {
	for (i = len-1; i > 0 && comes_before(w, tab[i-1]); --i)
	    tab[i] = tab[i-1];
	tab[i] = w;
	return len;
    } else
    	return len;
}


static void append(char **word, size_t *word_size, int *word_len, int c)
{
    if (*word_len == *word_size) {
        *word_size += 1024;
        *word = realloc(*word, *word_size);
        if (*word == NULL) {
            fprintf(stderr, "realloc failed: %d: %s", errno, strerror(errno));
            exit(EXIT_FAILURE);
        }
    }
    
    (*word)[*word_len] = c;
    ++(*word_len);
}


int main(int argc, char **argv)
{
    int c;
    int inside_word;
    char *word;
    size_t word_size;
    int word_len;
    static struct word *hashtab[MAX_HASH]; /* Compiler inits to NULL */
    struct word **toptab;
    int top_len;
    struct word *w;
    int i;
    int n;
    
    n = atoi(argv[1]);
    
    toptab = malloc(n * sizeof(struct word *));
    if (toptab == NULL) {
    	perror("malloc");
	exit(EXIT_FAILURE);
    }

    inside_word = 0;
    word_size = 1024;
    word = malloc(word_size);
    if (word == NULL) {
        fprintf(stderr, "malloc failed: %d: %s\n", errno, strerror(errno));
        exit(EXIT_FAILURE);
    }
    word_len = 0;
    while ((c = getchar()) != EOF) {
	c = tolower(c);
	if (inside_word) {
	    if (isalnum(c))
	        append(&word, &word_size, &word_len, c);
	    else {
	        append(&word, &word_size, &word_len, '\0');
		add_count(hashtab, word);
		inside_word = 0;
	    }
	} else {
	    if (isalnum(c)) {
		word_len = 0;
	        append(&word, &word_size, &word_len, c);
		inside_word = 1;
	    }
	}
    }
    
    if (inside_word) {
        append(&word, &word_size, &word_len, '\0');
	add_count(hashtab, word);
    }
    
    top_len = 0;

    for (i = 0; i < MAX_HASH; ++i) {
	for (w = hashtab[i]; w != NULL; w = w->next)
	    top_len = insert(toptab, top_len, n, w);
    }
    
    for (i = 0; i < top_len; ++i)
	printf("%ld %s\n", toptab[i]->count, toptab[i]->word);

    return 0;
}
