#!/usr/bin/python

import os, sys

INTERPRETERS = {
    ".py": ["python2.2", "python2.3"],
    ".sh": ["bash"],
    ".tcl": ["tclsh"],
    ".awk": ["gawk -f", "mawk -f"],
    ".rb": ["ruby1.8"],
}

INPUT_NAMES = ["simple", "longline", "huge", "binary"]

REPEATS = 5

def run_program(command, argv, stdin_filename, stdout_filename):
    before = os.times()
    stdin = os.open(stdin_filename, os.O_RDONLY)
    stdout = os.open(stdout_filename, os.O_WRONLY | os.O_CREAT, 0666)
    pid = os.fork()
    if pid == 0:
        # We are in child
        os.dup2(stdin, 0)
        os.dup2(stdout, 1)
        os.close(stdin)
        os.close(stdout)
        os.execvp(command, argv)
        sys.stderr.write("Execv failed")
        sys.exit(1)
    else:
        # We are in parent
        ret = os.waitpid(pid, 0)
        after = os.times()
        return after[2] - before[2], after[3] - before[3]

def run_with_inputs(results, key, command, argv):
    for input_name in INPUT_NAMES:
        print "Running '%s' with input %s:" % (" ".join(argv), input_name),
        for i in range(REPEATS):
            print i,
            sys.stdout.flush()
            user, system = run_program(command, 
                                       argv, 
                                       "input-" + input_name + ".txt", 
                                       "output-%s-%d-%s.txt" % 
                                            (input_name, i, key))
            results.add(key, input_name, i, user, system)
        print

class Results:

    def __init__(self):
        self.results = []
        
    def add(self, key, input_name, run_number, user, system):
        self.results.append((key, input_name, run_number, user, system))

    def compute(self):
        keys = []
        for result in self.results:
            if result[0] not in keys:
                keys.append(result[0])
        
        computed = {}
        
        for key in keys:
            computed[key] = {}
            for input_name in INPUT_NAMES:
                user = []
                system = []
                for result in self.results:
                    if result[0] == key and result[1] == input_name:
                        user.append(result[3])
                        system.append(result[4])
                user.sort()
                system.sort()
                computed[key][input_name] = (user[len(user)/2], 
                                             system[len(system)/2])

        return computed

    def maxlen(self, stringlist):
        return reduce(lambda a, b: max(a, b), map(lambda s: len(s), stringlist))

    def write_text(self, f, computed):
        keys = computed.keys()
        keys.sort()
        
        keyfmt = "%%%ds  " % self.maxlen(keys)
        namefmt = "%%%ds  " % self.maxlen(INPUT_NAMES)
        timefmt = "%.1f/%.1f"
        
        f.write(keyfmt % "")
        for input_name in INPUT_NAMES:
            f.write(namefmt % input_name)
        f.write("\n")
        f.write("\n")

        lines = 0
        for key in keys:
            f.write(keyfmt % key)
            for input_name in INPUT_NAMES:
                user, system = computed[key][input_name]
                t = timefmt % (user, system)
                f.write(namefmt % t)
            f.write("\n")
            lines += 1
            if lines == 3:
                f.write("\n")
                lines = 0
              
    def write_html(self, f, computed):
        keys = computed.keys()
        keys.sort()
        
        f.write("<table width='100%' border='1'>\n")
        f.write("<tr> <th>version</th>")
        for input_name in INPUT_NAMES:
            f.write(" <th>%s</th>" % input_name)
        f.write("</tr>\n")

        for key in keys:
            f.write("<tr> <td>%s</td>" % key)
            for input_name in INPUT_NAMES:
                user, system = computed[key][input_name]
                f.write(" <td>%.1f / %.1f</td>" % (user, system))
            f.write(" </tr>\n")
              
    def write(self):
        computed = self.compute()
        self.write_text(file("results.txt", "w"), computed)
        self.write_html(file("results.html", "w"), computed)

        f = open("results.dat", "w")
        for result in self.results:
            f.write("%s\t%s\t%d\t%f\t%f\n" % result)
        f.close()

def main():
    results = Results()
    for filename in sys.argv[1:]:
        basename, suffix = os.path.splitext(filename)
        if INTERPRETERS.has_key(suffix):
            for command in INTERPRETERS[suffix]:
                parts = command.split()
                if len(parts) > 1:
                    command = parts[0]
                    argv = parts
                else:
                    argv = [command]
                run_with_inputs(results, filename + "." + command, 
                                command, argv + [filename, "10"])
        else:
            run_with_inputs(results, filename, 
                            "./" + filename, ["./" + filename, "10"])
    results.write()

if __name__ == "__main__":
    main()
